#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w
use strict;
use PureMessage::Manager::Lang;
use PureMessage::Enduser::Request qw(%in);
use PureMessage::Enduser::Response;
use PureMessage::Enduser::UI::User;
use PureMessage::Lang qw(get_lang_map);
use ActiveState::DateTime qw(check_date);
use HTTP::Date qw(time2iso);

my $REQUEST          = new PureMessage::Enduser::Request;
my $r                = $REQUEST->get_response('', stderr_to_stdout => 0);
my $RESET_YEAR       = '1970';
my $RESET_DATE       = '1970-01-02';

# set language name space
PureMessage::Manager::Lang::set_namespace("EUUI/options");

$r->set_title('title_options');
$r->{debug} =0;

# user object does data retrieval
my $user;
if (! $r->authorize_user) {
    $r->send();
    exit;
} else {
    $user = $r->user();
}

# set the language
my $lang = $user->language();
$r->errors($user, $0, 'language', 'get_language_error');

# get the current saved settings
my %attributes = $user->get_attribute();
$r->errors($user, $0, 'get_attribute', 'set_attribute_error');

my %features = $user->get_features();
$r->errors($user, $0, 'get_features', 'set_attribute_error');
my $feature_vacation = $features{'option.vacation'};

if ((defined $in{'save'}) && ($in{'save'} eq lang_text_utf8($lang, 'button_save'))) {
    my $spam = ((defined $in{'spam'}) && ($in{'spam'} ne '')) ? 0 : 1;
    my $digest = ((defined $in{'digest'}) && ($in{'digest'} ne '')) ? 1 : 0;
    my $month = ((defined $in{'month'}) && ($in{'month'} ne '')) ? $in{'month'} : 1;
    my $day = ((defined $in{'day'}) && ($in{'day'} ne '')) ? $in{'day'} : 1;
    my $year = ((defined $in{'year'}) && ($in{'year'} ne '')) ? $in{'year'} : 0;
    my $language = $in{language};
    my $date;
    my $error = 0;
    if ($feature_vacation && (($month ne '__SELECT_MONTH__') || ($day ne '__SELECT_DAY__') || ($year ne '__SELECT_YEAR__'))) {
	if ($month eq '__SELECT_MONTH__') {
	    $r->raise_error(lang_text_utf8($lang, 'err_no_month'));
	    $error = 1;
	}
	if ($day eq '__SELECT_DAY__') {
	    $r->raise_error(lang_text_utf8($lang, 'err_no_day'));
	    $error = 1;
	}
	if ($year eq '__SELECT_YEAR__') {
	    $r->raise_error(lang_text_utf8($lang, 'err_no_year'));
	    $error = 1;
	}

	unless ($error) {
            if (! check_date($year, $month, $day)) {
		$r->raise_error(lang_text_utf8($lang, 'err_invalid_date'));
		$error = 1;
	    }
            $date = sprintf("%d-%02d-%02d 00:00:00", $year, $month, $day);
	}
    }

    $spam      = $spam ? 0 : 1;
    $digest    = $digest ? 0 : 1;

    # Date requires setting?
    unless ($error) {
        # Requires reset?
        if (_is_reset_date($date)) {
            $date = $RESET_DATE unless _is_reset_date($attributes{'vacation_till'});
        }
        # Requires modification?
        elsif ($attributes{'vacation_till'} ne $date) {
            $date = substr($date, 0, 10);
            if (_date_in_past($date)) {
                $r->raise_error(lang_text_utf8($lang, 'err_invalid_date'));
                $error = 1;
            }
        }
        else {
            $date = undef;
        }
    }

    unless ($error) {
        my %new_attributes;

        $new_attributes{'spam_optout'}   = $spam if $attributes{'spam_optout'} ne $spam;
        $new_attributes{'digest_optout'} = $digest if $attributes{'digest_optout'} ne $digest;
        $new_attributes{'vacation_till'} = $date if $date;
        $new_attributes{'language'}      = $language if $attributes{'language'} ne $language;

        if ( scalar(keys %new_attributes) ) {
	    foreach my $key ( keys %new_attributes ) {
		$attributes{$key} = $new_attributes{$key};
	    }

	    my $ret_value = $user->set_attribute(%new_attributes);
	    $r->errors($user, $0, 'set_attribute', 'set_attribute_error');
	    if ($ret_value) {
		$r->message(lang_text_utf8($language, 'mess_options_saved'));
	    } else {
		$r->raise_error(lang_text_utf8($lang, 'err_problem_saving'));
	    }
	}
    }
}

# set the language
$lang = $user->language();
$r->errors($user, $0, 'language', 'get_language_error');

my $language = $attributes{'language'};

my ($year, $month, $day);
if (!_is_reset_date($attributes{'vacation_till'})) {
    ($year, $month, $day) = _date_part($attributes{'vacation_till'});
}

my $spam   = $attributes{'spam_optout'};
my $digest = !$attributes{'digest_optout'};

if ((! $features{'option.spam_optout'}) && (!  $features{'option.digest_optout'}) 
    && (! $features{'option.vacation'}) && (! $features{'language.preference'})) {
    $r->raise_error(lang_text_utf8($lang, 'err_feature_disabled'));
    $r->send;
    exit;
}
my $feature_preferences = $features{'option.spam_optout'} || $features{'option.digest_optout'};
my $feature_spam = $features{'option.spam_optout'};
my $feature_digest = $features{'option.digest_optout'};
my $feature_language = $features{'language.preference'};

my %settings = $user->get_settings();
$r->errors($user, $0, 'get_settings', 'set_attribute_error');
my $hold_days = $settings{'quarantine_expiry_days'};
# if we get -1 (which means it is not set in the crontab), disable this features
if ($hold_days == -1) {
    $feature_vacation = 0;
} else {
    $hold_days *= 2;
}

my %month_names = ('01' => lang_text_utf8($lang, 'mon_jan'),
		   '02' => lang_text_utf8($lang, 'mon_feb'),
		   '03' => lang_text_utf8($lang, 'mon_mar'),
		   '04' => lang_text_utf8($lang, 'mon_apr'),
		   '05' => lang_text_utf8($lang, 'mon_may'),
		   '06' => lang_text_utf8($lang, 'mon_jun'),
		   '07' => lang_text_utf8($lang, 'mon_jul'),
		   '08' => lang_text_utf8($lang, 'mon_aug'),
		   '09' => lang_text_utf8($lang, 'mon_sep'),
		   '10' => lang_text_utf8($lang, 'mon_oct'),
		   '11' => lang_text_utf8($lang, 'mon_nov'),
		   '12' => lang_text_utf8($lang, 'mon_dec'),
		   );

my @loop_month;
foreach my $mon (sort (keys %month_names)) {
    if ((defined $month) && ($mon eq $month)) {
	push (@loop_month, {'month_num' => $mon, 'selected' => 1, 'month_long' => $month_names{$mon}});
    } else {
	push (@loop_month, {'month_num' => $mon, 'selected' => 0, 'month_long' => $month_names{$mon}});
    }
}
my @loop_day;
for (my $i = 1; $i <= 31; $i++) {
    if ((defined $day) && ($i == $day)) {
	push (@loop_day, {'day' => $i, 'selected' => 1});
    } else {
	push (@loop_day, {'day' => $i, 'selected' => 0});
    }
}

my ($curr_year, @junk) = _date_part(time2iso());
my $end_year = $curr_year + 1;

my @loop_year;
for my $yr ($curr_year .. $end_year) {
    push (@loop_year, { 'year' => $yr, 
			'selected' => (defined $year and $yr eq $year) ? 1 : 0
		    });
}

my %lang_map = get_lang_map();
my @option_language;
foreach my $key (keys(%lang_map)) {
    my $lang;
    if ( (defined($language)) && ($language eq $key)) {
	$lang = {'selected' => 1, 'language' => $key, 'language_translated' => $lang_map{$key}->{'trans_lang'}};
    } else {
	$lang = {'selected' => 0, 'language' => $key, 'language_translated' => $lang_map{$key}->{'trans_lang'}};
    }   
    push (@option_language, $lang);
}

# Determines formatting in the template file
# ie. Date should be year/month/date in Japanese, not month/day/year
my $lang_jp = ($lang eq 'jp');

my $content_tmpl = $r->new_tmpl('content_options.html');
$content_tmpl->param('title_preferences' => lang_text_utf8($lang, 'title_preferences'),
		     'spam_message' => lang_text_utf8($lang, 'spam_message'),
		     'digest_message' => lang_text_utf8($lang, 'digest_message'),
		     'title_hold' => lang_text_utf8($lang, 'title_hold'),
		     'hold_message' => lang_text_utf8($lang, 'hold_message', $hold_days),
		     'hold_until' => lang_text_utf8($lang, 'hold_until', $hold_days),
		     'button_save' => lang_text_utf8($lang, 'button_save'),
		     'button_cancel' => lang_text_utf8($lang, 'button_cancel'),
		     'feature_preferences' => $feature_preferences,
		     'feature_spam' => $feature_spam,
		     'feature_digest' => $feature_digest,
		     'feature_vacation' => $feature_vacation,
		     'spam' => $spam,
		     'digest' => $digest,
		     'select_month' => lang_text_utf8($lang, 'select_month'),
		     'option_month' => \@loop_month,
		     'select_day' => lang_text_utf8($lang, 'select_day'),
		     'option_day' => \@loop_day,
		     'select_year' => lang_text_utf8($lang, 'select_year'),
		     'option_year' => \@loop_year,
		     'feature_language' => $feature_language,
		     'title_language' => lang_text_utf8($lang, 'title_language'),
		     'option_language' => \@option_language,
                     'lang_jp' => $lang_jp,
		     );

$r->add_part($content_tmpl);
$r->send;

sub _date_in_past {
    my $setiso = shift;
    my $nowiso = time2iso();

    my ($now)  = split(' ', $nowiso); $now =~ s/-//g;
    my ($set)  = split(' ', $setiso); $set =~ s/-//g;

    return $set < $now;
}

sub _is_reset_date {
    my $date = shift;
    return 1 unless $date;
    my ($year, @junk) = _date_part($date);
    return 1 if $year == $RESET_YEAR;
    return 0;
}

sub _date_part {
    my $date = shift;
    return '' unless $date;
    return (_date_components($date))[0..2];
}

sub _date_components {
    my $date = shift;
    $date =~ s/ /-/g;
    my @tm = split('-', $date);
    return @tm;
}
