#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w
use strict;
use Mail::Address;
use PureMessage::Manager::Lang;
use PureMessage::Enduser::Request qw(%in);
use PureMessage::Enduser::Response;
use PureMessage::Enduser::UI::User;
my $REQUEST = new PureMessage::Enduser::Request;
my $r = $REQUEST->get_response('', stderr_to_stdout => 0);

# set language name space
PureMessage::Manager::Lang::set_namespace("EUUI/senders");

# sender types must be either 'approved' or 'blocked'
# - defaults to 'approved'
my $sender_type = $in{'sender_type'};
if ((! defined $sender_type) || (($sender_type ne 'approved') && ($sender_type ne 'blocked'))) {
    $sender_type = 'approved';
}

# if there is not field to sort by, the order is irrelevant   
my ($order_ascending, $order_descending);
my $sort_order = $in{'sort_order'};
my $cgi_sort_order;
if ((defined $sort_order) && ($sort_order eq 'ascending')) {
    $sort_order = 0;
    $cgi_sort_order = 'ascending';
    $order_ascending = 1;
    $order_descending = 0;
} elsif ((defined $sort_order) && ($sort_order eq 'descending')) {
    $sort_order = 1;
    $cgi_sort_order = 'descending';
    $order_ascending = 0;
    $order_descending = 1;
} else {
    $sort_order = 0;
    $cgi_sort_order = 'ascending';
    $order_ascending = 1;
    $order_descending = 0;
}

my ($title, $list_id, $buttons_blocked, $buttons_deleted);
if ((defined $sender_type) && ($sender_type eq 'approved')) {
    $title = 'title_approved_senders';
    $list_id = 'whitelist';
} elsif ((defined $sender_type) && ($sender_type eq 'blocked')) {
    $title = 'title_blocked_senders';
    $list_id = 'blacklist';
} else {
    $title = 'title_approved_senders';
    $list_id = 'whitelist';
}

$r->set_title($title);
$r->{debug} =0;

# user object does data retrieval
my $user;
if (! $r->authorize_user) {
    $r->send();
    exit;
} else {
    $user = $r->user();
}

# set the language
my $lang = $user->language();
$r->errors($user, $0, 'language', 'get_language_error');

my %features = $user->get_features();
$r->errors($user, $0, 'get_features', 'excessive_list_items');
if (($list_id eq 'whitelist') && (! $features{'list.white'})) {
    $r->raise_error(lang_text_utf8($lang, 'feature_disabled'));
    $r->send;
    exit;
}
if (($list_id eq 'blacklist') && (! $features{'list.black'})) {
    $r->raise_error(lang_text_utf8($lang, 'feature_disabled'));
    $r->send;
    exit;
}

my @senderids;
if ((ref($in{'selectsingle'})) && (ref($in{'selectsingle'}) eq 'ARRAY')) {
    @senderids = @{$in{'selectsingle'}};
} else {
    push (@senderids, $in{'selectsingle'}) if defined $in{'selectsingle'};
}

if ((defined $in{'delete'}) && ($in{'delete'} eq lang_text_utf8($lang, 'delete_sender_button'))) {
    my $senders = $user->get_items($list_id);
    $r->errors($user, $0, 'get_items', 'excessive_list_items');
    if (!$senders or (ref($senders) eq 'ARRAY' and scalar(@$senders) == 0) ) {
	if ($sender_type eq 'approved') {
	    $r->raise_error(lang_text_utf8($lang, 'no_approved_senders'));
	}
	if ($sender_type eq 'blocked') {
	    $r->raise_error(lang_text_utf8($lang, 'no_blocked_senders'));
	}
    } else {
	if ($order_ascending) {
	    @$senders = sort { lc($a) cmp lc($b) } @$senders;
	} elsif ($order_descending) {
	    @$senders = sort { lc($b) cmp lc($a) } @$senders;
	} else {
	    @$senders = sort { lc($a) cmp lc($b) } @$senders;
	}
	# - each time splice is used on the array the index moves down by one
	#   - this is taken into account by the index $i and incrementing $i
	#     for each splice done
	my $i = 0;
	foreach my $idx (@senderids) {
	    splice(@$senders, ($idx - $i), 1);
	    $i++;
	}
	my $ret_value = $user->set_items($list_id, $senders);
	$r->errors($user, $0, 'set_items', 'excessive_list_items');
	if (! $ret_value) {
	    $r->raise_error(lang_text_utf8($lang, 'could_not_delete'));
	}
	if ($sender_type eq 'approved') {
	    $r->message(lang_text_utf8($lang, 'deleted_from_approved_senders'));
	}
	if ($sender_type eq 'blocked') {
	    $r->message(lang_text_utf8($lang, 'deleted_from_blocked_senders'));
	}
    }
}

if ((defined $in{'add'}) && ($in{'add'} eq lang_text_utf8($lang, 'button_add_sender'))) {
    my %features = $user->get_features();
    $r->errors($user, $0, 'get_features', 'excessive_list_items');
    my $list_wildcard = $features{'list.wildcard'};
    my $content_tmpl = $r->new_tmpl('content_add_sender.html');
    $content_tmpl->param('add_sender_title' => lang_text_utf8($lang, 'add_sender_title'),
			 'valid_address_form' => lang_text_utf8($lang, 'valid_address_form'),
			 'wildcard_use' => lang_text_utf8($lang, 'wildcard_use'),
			 'title_address' => lang_text_utf8($lang, 'title_address'),
			 'button_add_sender' => lang_text_utf8($lang, 'button_add_sender'),
			 'button_cancel' => lang_text_utf8($lang, 'button_cancel'),
			 'content_sender_type' => $sender_type,
			 'list_wildcard' => $list_wildcard,
			 );
    $r->add_part($content_tmpl);
    $r->send;
    exit;
}

if ((defined $in{'addaddr'}) && ($in{'addaddr'} eq lang_text_utf8($lang, 'button_add_sender'))) {
    my %features = $user->get_features();
    $r->errors($user, $0, 'get_features', 'excessive_list_items');
    my $list_wildcard = $features{'list.wildcard'};
    my ($new_address) = Mail::Address->parse($in{'add_address'});
    my $add_address = defined($new_address) ? $new_address->format() : undef;
    # damn Keiths code get's around a lot
    # borrowed from Keith's code
    if (!$add_address
        or ((! $list_wildcard) && ($add_address !~ m/\A\w[-.\w]*\@[-\w]+(?:\.[-\w]+)*\Z/x))
        or (($list_wildcard) && ($add_address !~ m/\A((\*|\w)([-.\*]|[-.\w])*)?\@[-\w]+(?:\.[-\w]+)*\Z/x))) {
	$r->raise_error(lang_text_utf8($lang, 'invalid_address'));
	my $content_tmpl = $r->new_tmpl('content_add_sender.html');

	$content_tmpl->param('add_sender_title' => lang_text_utf8($lang, 'add_sender_title'),
			     'valid_address_form' => lang_text_utf8($lang, 'valid_address_form'),
			     'wildcard_use' => lang_text_utf8($lang, 'wildcard_use'),
			     'title_address' => lang_text_utf8($lang, 'title_address'),
			     'button_add_sender' => lang_text_utf8($lang, 'button_add_sender'),
			     'button_cancel' => lang_text_utf8($lang, 'button_cancel'),
			     'content_sender_type' => $sender_type,
			     'list_wildcard' => $list_wildcard,
			     'address_value' => $in{'add_address'},
			     );
	$r->add_part($content_tmpl);
	$r->send;
	exit;
    } 

    my $senders = $user->get_items($list_id);
    $r->errors($user, $0, 'get_items', 'excessive_list_items');
    my %seen_senders;
    my @unique_senders;
    foreach my $sender (@$senders) {
	unless ($seen_senders{$sender}) {
	    $seen_senders{$sender} = 1;
	    push(@unique_senders, $sender);
	}
    }
    $senders = \@unique_senders;
    push (@$senders, $add_address) unless ($seen_senders{$add_address});
    my $ret_value = $user->set_items($list_id, $senders);
    $r->errors($user, $0, 'set_items', 'excessive_list_items');
    if (not $ret_value) {
	$r->raise_error(lang_text_utf8($lang, 'could_not_save'));
    } else {
	if ($sender_type eq 'approved') {
	    $r->message(lang_text_utf8($lang, 'approved_sender_added'));
	}
	if ($sender_type eq 'blocked') {
	    $r->message(lang_text_utf8($lang, 'blocked_sender_added'));
	}
    }
}

my $senders = $user->get_items($list_id);
$r->errors($user, $0, 'get_items', 'excessive_list_items');

if (!$senders or (ref($senders) eq 'ARRAY' and scalar(@$senders) == 0) ) {
    my %features = $user->get_features();
    $r->errors($user, $0, 'get_features', 'excessive_list_items');
    my $list_wildcard = $features{'list.wildcard'};
    if ($sender_type eq 'approved') {
	$r->add_part("<p>" . lang_text_utf8($lang, 'no_approved_senders') . "</p>\n");
    }
    if ($sender_type eq 'blocked') {
	$r->add_part("<p>" . lang_text_utf8($lang, 'no_blocked_senders') . "</p>\n");
    }
    my $content_tmpl = $r->new_tmpl('content_add_sender.html');
    $content_tmpl->param('add_sender_title' => lang_text_utf8($lang, 'add_sender_title'),
			 'valid_address_form' => lang_text_utf8($lang, 'valid_address_form'),
			 'wildcard_use' => lang_text_utf8($lang, 'wildcard_use'),
			 'title_address' => lang_text_utf8($lang, 'title_address'),
			 'button_add_sender' => lang_text_utf8($lang, 'button_add_sender'),
			 'button_cancel' => lang_text_utf8($lang, 'button_cancel'),
			 'content_sender_type' => $sender_type,
			 'list_wildcard' => $list_wildcard,
			 );
    $r->add_part($content_tmpl);
    $r->send;
    exit;
}

if ($order_ascending) {
    @$senders = sort { lc($a) cmp lc($b) } @$senders;
} elsif ($order_descending) {
    @$senders = sort { lc($b) cmp lc($a) } @$senders;
} else {
    @$senders = sort { lc($a) cmp lc($b) } @$senders;
}

my $pages = int(@$senders / 20);
if ((@$senders % 20) != 0) {
    $pages++;
}

my $page = ((defined $in{'page'}) && ($in{'page'} > 0)) ? $in{'page'} : 1;
$page = ($page > $pages) ? $pages : $page;

if (((defined $in{'next'}) && ($in{'next'} ne '')) && ($page < $pages)) {
    $page++;
}
if (((defined $in{'previous'}) && ($in{'previous'} ne '')) && ($page > 1)) {
    $page--;
}

if ((defined $in{'top_page'}) && ($in{'top_page'} =~ /\d/)) {
    $page = $in{'top_page'};
}

if ((defined $in{'bottom_page'}) && ($in{'bottom_page'} =~ /\d/)) {
    $page = $in{'bottom_page'};
}

my $previous = ($page > 1) ? 1 : 0;
my $next = ($page < $pages) ? 1 : 0;

my @pgs;
for (my $i = 1; $i <= $pages; $i++) {
    next if ($i == $page);
    push(@pgs, {'page_label' => lang_text_utf8($lang, 'page_label'),
		'content_sender_page_number' => $i});
}

my $start = (($page - 1) * 20);
my $end = $start + 19;
$end = ($end > $#$senders) ? $#$senders : $end;

my @content_senders;
for (my $i = $start; $i <= $end; $i++) {
    my $temp_sender = $senders->[$i];
    if ($senders->[$i] =~ s/^(?:\*\*)?\@/lang_text_utf8($lang, 'all_email_from').' '/e) {
	push (@content_senders, {'content_sender_list_value' => $i,
				 'content_sender_list_sender' => $senders->[$i] . " (" . $temp_sender . ")"},
	      );
    } else {
	push (@content_senders, {'content_sender_list_value' => $i,
				 'content_sender_list_sender' => $senders->[$i]},
	      );
    }
}

my $content_tmpl = $r->new_tmpl('content_sender.html');

$content_tmpl->param('sender_header_name' => lang_text_utf8($lang, 'sender_header_name'),
		     'button_add_sender' => lang_text_utf8($lang, 'button_add_sender'),
		     'delete_sender_button' => lang_text_utf8($lang, 'delete_sender_button'),
		     'content_sender_type' => $sender_type,
		     'content_sender_hidden_page' => $page,
		     'sort_order' => $cgi_sort_order,
		     'order_ascending' => $order_ascending,
		     'order_descending' => $order_descending,
		     'sender_page_info' => lang_text_utf8($lang, 'sender_page_info', ($start + 1), ($end + 1), scalar(@$senders), $page, $pages),
		     'page_jump' => lang_text_utf8($lang, 'page_jump'),
		     'content_sender_page_previous' => $previous,
		     'content_sender_page_next' => $next,
		     'content_sender_page' => \@pgs,
		     'content_sender_list' => \@content_senders);

$r->add_part($content_tmpl);
$r->send;
