package PureMessage::Enduser::UI::Config;
use strict;
use PureMessage::Config qw(expand_filename);
use PureMessage::Resources::Manager;
use PureMessage::Enduser::UI::Config::Validator;
use base qw(Exporter);

our @EXPORT_OK = qw( %config %enduser_ui_config);
our %config;
our $object;

our $enduser_ui_config; # alias that doesn't clash with other %configs
*enduser_ui_config =\%config;


our $RESOURCE_MANAGER;

sub refresh() {
    require PureMessage::Config::Validator;

    unless($RESOURCE_MANAGER) {
      $RESOURCE_MANAGER = PureMessage::Resources::Manager->new();
    }
    
    my $res = $RESOURCE_MANAGER->find('enduser_ui_config');
    unless($res) {
        $res = $RESOURCE_MANAGER->add('enduser_ui_config');
    }
    die "Unable to initialize enduser_ui_config resource. Have the default resources been initialized?"
        unless $res;
    
    $object = $res->native();
 
    PureMessage::Config::Validator->define_core_validators($object);
    PureMessage::Enduser::UI::Config::Validator->validate($object);
    $object->parse_file(expand_filename("etc/enduser/enduser_ui.conf"));
    %config = $object->as_hash;
    foreach (qw( log_dir)) {
      $config{$_} = $PureMessage::Config::config{$_};
    }
}

refresh();

=item edit()

 Will open this config as a writeable  ActiveState::Config object
 and return it. This can die when configuration file has invalid
 syntax.

=cut

sub edit {
    
   unless($RESOURCE_MANAGER) {
     $RESOURCE_MANAGER = PureMessage::Resources::Manager->new();

   }
   my $res = $RESOURCE_MANAGER->find('enduser_ui_config');
   unless($res) {
       $res = $RESOURCE_MANAGER->add('enduser_ui_config');
   }
   die "Unable to initialize enduser_config resource. Have the default resources been initialized?"
       unless $res;
    
   my $p = $res->native(writable => 1);
 
   PureMessage::Config::Validator->define_core_validators($p);
   PureMessage::Enduser::UI::Config::Validator->validate($p);
   $p->parse_file(expand_filename("etc/enduser/enduser_ui.conf"),writable => 1);
   return $p;
}

1;
