package PureMessage::Enduser::UI::User::local;

use PureMessage::RPC::ClientStub;
use PureMessage::Config;
use PureMessage::Enduser::UI::Config qw(%config);
use PureMessage::Manager::Lang qw(mtext text );
use Data::Dumper;
use strict;
use warnings;

our $DEFAULT_LANGUAGE = $PureMessage::Config::config{language} || 'en';

use base 'PureMessage::Enduser::UI::User::rpc';

sub have_cache {
  return 0;
}

sub _init_rpc {
  my $self = shift;
  $self->{rpc_client} = PureMessage::RPC::ClientStub->new();
}

# internal method using to make the actual rpc calls
sub _rpc {
  my ($self,$call,@args) = @_;
  return if $self->error();
  
  my $client = $self->{rpc_client};
  my $keep_lang = PureMessage::Manager::Lang::get_namespace(); 
  PureMessage::Manager::Lang::set_namespace("/RPC/EnduserUI"); # for our lang entries
  $client->call('EnduserUILocal',$call,@args);
  PureMessage::Manager::Lang::set_namespace($keep_lang); 
  $client->wait();
  if($client->error()) {
     $self->set_rpc_error($client->error());
     return;
  } else {
    # we have rpc data
    my @data = $client->data();
    my ($results,$error) = @data;
    if ($results) {
        $self->cache_results($results); # even if there was an error we should inspect the return for data
        if ( scalar( keys(%$error) ) ) {
           $self->{extended_error} = $error;
           $self->set_rpc_error(join(' ',map{$error->{$_}->{errorstr}} keys %$error));
           return;
        } else {
          # we got data, make sure it's in the right format
          if(exists($results->{$call})) {
            my $r = $results->{$call};
            return $self->massage_results($call,$r);
          } else {
            return; # we got no data back
          }
        }
    } else {
      return @data;
    }
  }

}

1;

