package PureMessage::RPC::ClientStub;
use PureMessage::Manager::Lang;
use PureMessage::Version qw($PREFIX);
use PureMessage::Enduser::UI::Config qw(%config);
use PureMessage::Logger qw( LOG_DEBUG LOG_INFO LOG_NOTICE LOG_WARNING LOG_ERR LOG_CRIT LOG_ALERT LOG_EMERG);

my $LOG_LOCATION = "$PREFIX/var/log/manager/eui_local_log";

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my %opts = @_;
    return bless( \%opts, $class);
}

sub _init_log {
  my $self = shift;
  return $self->{'log'} if defined($self->{'log'}) and $self->{'log'};
  $self->{'log'} = PureMessage::Logger->new( ident => 'eui_local', log_to=>$LOG_LOCATION,debug_level => $config{debug_level} || 0);
  $self->{'log'};
}

sub _log {
  my ($self,$log,$PRIORITY,@msg) = @_;
  my $logo = $self->_init_log();
  $logo->log($PRIORITY,join(' ',@msg));
  
}

sub logger {
  my $self = shift;
  return $self->_logger();
}

sub _logger {
  my $self = shift;
  return $self->{'log'} ? $self->{'log'} : $self->_init_log();
}

sub log {
  my $self = shift;
  $self->_log('audit_log',LOG_INFO,@_);
}

sub log_error {
  my $self = shift;
  $self->log(@_);
}

sub wait {
}

sub error {
}

sub data {
   my $self = shift;
   return ($self->{data},$self->{errors});
}


sub call {
    my $self = shift;
    my $true_class = shift || 'PureMessage::RPC::Response';
    my $method = shift || 'doc';
    my $class = "PureMessage::RPC::Response::$true_class" if $true_class !~ /::/;
    my @data = @_;
    $self->{data} = undef;
    $self->{errors} = undef;
    my $r;
    {
      no warnings;
      eval "require $class";
    };
    if ( $@ ) {
        my $err = $@;
        $self->log("Service not found: $err");
        $r->raise_error("Service not found: $err");
        die "Service not found: $err";
    }
    else {
            no warnings qw(once); # turn off only used once warning for OLDOUT
           
            $r = $class->new( _request => $self, buffer => 0);
            $r->{REQUEST} = $self;
            unless($r->isa('PureMessage::RPC::Response')) {
             die "Invalid Package '$class'";
            }
            if ( $r->can($method) ) {
                #$r->log("Dispatched $method with " . Dumper(\@data));
                eval {$r->$method(@data)};
                my $err = $@;
                if ( $err ) {
                   $r->raise_error("RPC call failed: $err") unless $r->dispatched_errors();
                   $r->log("RPC call failed: $err");
                };
                $self->{errors} = $r->get_error();
                $self->{data} = $r->get_data();
     
             } else {
                $self->{errors} =  "Invalid method: $method\n";
                $r->log("RPC call invalid method: $method\n");
             }
        }
}

1;


