package PureMessage::RPC::Response::EnduserUILocal;

use PureMessage::Manager::Lang qw(text mtext);
use base 'PureMessage::RPC::Response::Enduser';
use Data::Dumper;
use strict;

sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self = $class->SUPER::new(@_);
  return unless $self;
  $self->{DATA} = undef;
  $self->{ERRORS} = undef;
  return bless($self,$class);
}

sub dispatch {
  my ($self,@data) = @_;
  # figureout who called us
  my @stack = caller(1);
  my $caller = $stack[3] =~ /:{2}?([^:]+)$/ ? $1 : 'anon';
  $self->named_dispatch($caller,@data);
}

sub dispatched_errors {
  my $self = shift;
  return keys(%{$self->{ERRORS}}) ? 1 : 0;
}

sub named_dispatch {
   my ($self,$name,@data) = @_;
   if(exists($self->{DATA}->{$name})) {
      if(ref($data[0]) eq 'ARRAY') {
          foreach my $ref (@data) {
              push(@{$self->{DATA}->{$name}},@$ref);
          }
      } else {
          push(@{$self->{DATA}->{$name}},@data);
      }
   } else {
      if(ref($data[0]) eq 'ARRAY') {
          foreach my $ref (@data) {
              push(@{$self->{DATA}->{$name}},@$ref);
          }
      } else {
          $self->{DATA}->{$name} = \@data;
      }
 
   }
   1;
}

sub raise_detailed_error {
  my ($self,$name,%extra) = @_;
  my $i=0;
  my $text = mtext($name,grep { $i++ % 2 } %extra);
  $self->log("dispatched error: " . $text);
  $self->{ERRORS}->{$name} = \%extra;
}

sub get_data {
    my $self = shift;
    return {} unless ref($self->{DATA}) and keys(%{$self->{DATA}});
    my %d = %{$self->{DATA}};
    %{$self->{DATA}} = ();
    return \%d;
}

sub get_error {
    my $self = shift;
    return {} unless ref($self->{ERRORS}) and keys(%{$self->{ERRORS}});
    my %e = %{$self->{ERRORS}};
    %{$self->{ERRORS}} = ();
    return \%e;
}

1;



